<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE xsl:stylesheet SYSTEM "DetailedReport.dtd">
<xsl:stylesheet version="2.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:x="http://www.greenhatsoftware.com/functions/DetailedReport.xsl"
	xmlns:gfn="http://www.greenhatsoftware.com/Functions.xsl"
	exclude-result-prefixes="x gfn">

	<xsl:output method="html" encoding="UTF-8" indent="yes" />

	<xsl:include href="Utils/Variables.xsl" />
	<xsl:include href="Utils/DateTime.xsl" />
	<xsl:include href="Utils/A3Message.xsl" />
	<xsl:include href="Utils/Functions.xsl" />
	<xsl:include href="plugableActions.xsl" />

	<!-- externalisations -->
	<xsl:variable name="detailedReport.summarySectionHeader">
		<xsl:text>&detailedReport.summarySectionHeader;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.title">
		<xsl:text>&detailedReport.title;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.version">
		<xsl:text>&detailedReport.version;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.test">
		<xsl:text>&detailedReport.test;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.environment">
		<xsl:text>&detailedReport.environment;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.overallStatus">
		<xsl:text>&detailedReport.overallStatus;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.warnings">
		<xsl:text>&detailedReport.warnings;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.errors">
		<xsl:text>&detailedReport.errors;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.initialise">
		<xsl:text>&detailedReport.initialise;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.testSteps">
		<xsl:text>&detailedReport.testSteps;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.tearDown">
		<xsl:text>&detailedReport.tearDown;</xsl:text>
	</xsl:variable>
	<xsl:variable name="detailedReport.environmentHeading">
		<xsl:text>&detailedReport.environmentHeading;</xsl:text>
	</xsl:variable>

	<xsl:param name="locale" />

	<xsl:template match="/">
		<html>
			<xsl:attribute name="lang"><xsl:value-of select="$locale"/></xsl:attribute>
			<xsl:attribute name="xml:lang"><xsl:value-of select="$locale"/></xsl:attribute>
			<head>
				<title>
					<xsl:value-of select="x:getTitle()" />
				</title>
				<link rel="stylesheet" type="text/css" href="resourcestyle.css" />
				<link rel="stylesheet" type="text/css" href="iprocessstyles.css" />
			</head>
			<body>
				<xsl:apply-templates select="*" />
			</body>
		</html>
	</xsl:template>

	<xsl:template match="Test">
		<xsl:call-template name="generateReportTitle" />
		<table class="faux">
			<xsl:call-template name="reportTestInfo" />
		</table>
		<xsl:call-template name="addSectionHeader">
			<xsl:with-param name="header"
				select="$detailedReport.summarySectionHeader" />
		</xsl:call-template>
		<xsl:call-template name="createSummaryTable" />
		<xsl:call-template name="processTestDataSet" />
		<xsl:call-template name="processEnvironment" />
		<xsl:apply-templates select="*" />
	</xsl:template>

	<xsl:function name="x:getTitle">
		<xsl:sequence select="$detailedReport.title" />
	</xsl:function>

	<!-- Returns the correct indentation for the provided node. This is basically
		a count of ancestors of the node - 1 if the node is the first action of a
		failure path node - ie the failing action -->
	<xsl:function name="x:indentation">
		<xsl:param name="node" />
		<xsl:value-of
			select="if(not($node/preceding-sibling::*) and $FAILURE_PATHS/*[name() = name($node/..)]) then count($node/ancestor::*) - 1 else count($node/ancestor::*)" />
	</xsl:function>

	<xsl:template name="copyAttribute">
		<xsl:param name="node" select="current()" />
		<xsl:param name="attribute" />
		<xsl:param name="outputAttribute" />
		<xsl:param name="withTitle" select="true()" />
		<xsl:param name="class" />
		<xsl:param name="formatNewLines" />
		<xsl:choose>
			<xsl:when test="$node/@*[name()=$attribute]">
				<xsl:for-each select="$node/@*[name()=$attribute]">
					<xsl:if test="$withTitle">
						<th>
							<xsl:value-of select="$outputAttribute" />
							<xsl:value-of select="string(':')" />
						</th>
					</xsl:if>
					<td>
						<xsl:if test="string-length(string($class)) != 0">
							<xsl:attribute name="class">
                                <xsl:value-of select="$class" />
                            </xsl:attribute>
						</xsl:if>
						<xsl:choose>
							<xsl:when test="string-length(string(current())) != 0">
								<xsl:choose>
									<xsl:when test="$formatNewLines=false()">
										<xsl:value-of select="." />
									</xsl:when>
									<xsl:otherwise>
										<xsl:call-template name="formatNewLines">
											<xsl:with-param name="text" select="current()" />
										</xsl:call-template>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="$NOT_APPLICABLE" />
							</xsl:otherwise>
						</xsl:choose>
					</td>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="$withTitle">
					<th>
						<xsl:value-of select="concat($outputAttribute,':')" />
					</th>
				</xsl:if>
				<td>
					<xsl:value-of select="$NOT_APPLICABLE" />
				</td>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="getAs3DP">
		<xsl:param name="val" />
		<xsl:choose>
			<xsl:when test="contains($val, string('.'))">
				<xsl:value-of
					select="concat(substring-before($val,'.'),'.',substring(substring-after($val, string('.')),1,3 ))" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$val" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="getChildCountByAttribute">
		<xsl:param name="name" />
		<xsl:param name="value" />
		<xsl:param name="incAttribute" />
		<xsl:param name="incValue" />
		<xsl:if test="string-length(string($name)) != 0">
			<xsl:choose>
				<xsl:when
					test="string-length(string($incAttribute)) != 0 and string-length(string($incValue)) != 0">
					<xsl:value-of
						select="count(child::*[attribute::*[name()=$name] and contains(attribute::*[name()=$name], $value) and attribute::*[name()=$incAttribute] and contains(attribute::*[name()=$incAttribute], $incValue)])" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of
						select="count(child::*[attribute::*[name()=$name] and contains(attribute::*[name()=$name], $value)])" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>

	<xsl:template name="reportCommonInfo">
		<xsl:param name="validation" select="false()" />
		<xsl:param name="host" select="false()" />
		<xsl:choose>
			<xsl:when test="parent::*/parent::Filter">

				<!-- this is a bit of a hack. For each is the only way to change the
					context node. When dealing with a filtered action the parent filter element
					has the information eg iterationStatus. -->

				<xsl:for-each select="parent::*/parent::Filter">
					<xsl:call-template name="reportDescription" />
					<tr>
						<xsl:call-template name="reportStartTime" />
						<xsl:call-template name="reportEndTime" />
					</tr>
					<tr>
						<xsl:call-template name="reportExecutionTime" />
						<xsl:if test="$host">
							<xsl:call-template name="reportHostID" />
						</xsl:if>
						<xsl:if test="$validation">
							<xsl:call-template name="reportValidationError" />
						</xsl:if>
					</tr>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="reportDescription" />
				<tr>
					<xsl:call-template name="reportStartTime" />
					<xsl:call-template name="reportEndTime" />
				</tr>
				<tr>
					<xsl:call-template name="reportExecutionTime" />
					<xsl:if test="$host">
						<xsl:call-template name="reportHostID" />
					</xsl:if>
					<xsl:if test="$validation">
						<xsl:call-template name="reportValidationError" />
					</xsl:if>
				</tr>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="reportDescription">
		<xsl:param name="description" select="'business'" />
		<xsl:if test="@*[name()=$description]">
			<tr>
				<xsl:call-template name="reportInfo">
					<xsl:with-param name="value" select="@*[name()=$description]" />
					<xsl:with-param name="tdSpan" select="4" />
				</xsl:call-template>
			</tr>
		</xsl:if>
	</xsl:template>

	<xsl:template name="reportInfo">
		<xsl:param name="name" />
		<xsl:param name="value" />
		<xsl:param name="tdSpan" />
		<xsl:param name="href" />
		<xsl:param name="class" />
		<xsl:if test="string-length(string($name)) !=0 ">
			<th>
				<xsl:value-of select="$name" />
				<xsl:value-of select="string(':')" />
			</th>
		</xsl:if>
		<xsl:call-template name="reportInfoData">
			<xsl:with-param name="tdSpan" select="$tdSpan" />
			<xsl:with-param name="href" select="$href" />
			<xsl:with-param name="value" select="$value" />
			<xsl:with-param name="class" select="$class" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="reportInfoData">
		<xsl:param name="tdSpan" />
		<xsl:param name="href" />
		<xsl:param name="value" />
		<xsl:param name="class" />
		<td>
			<xsl:if test="string-length(string($tdSpan))!=0">
				<xsl:attribute name="colspan">
                    <xsl:value-of select="$tdSpan" />
                </xsl:attribute>
			</xsl:if>
			<xsl:if test="string-length(string($class))!=0">
				<xsl:attribute name="class">
                    <xsl:value-of select="$class" />
                </xsl:attribute>
			</xsl:if>
			<xsl:choose>
				<xsl:when test="string-length(string($href))!=0">
					<a>
						<xsl:attribute name="href">
                            <xsl:value-of select="$href" />
                        </xsl:attribute>
						<xsl:call-template name="formatNewLines">
							<xsl:with-param name="text" select="$value" />
						</xsl:call-template>
					</a>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="formatNewLines">
						<xsl:with-param name="text" select="$value" />
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</td>
	</xsl:template>

	<xsl:template name="reportInfoPass">
		<xsl:param name="name" />
		<xsl:param name="value" />
		<xsl:param name="tdSpan" />
		<xsl:param name="href" />
		<xsl:if test="string-length($name) !=0 ">
			<th class="pass">
				<xsl:value-of select="concat($name,':')" />
			</th>
		</xsl:if>
		<xsl:call-template name="reportInfoData">
			<xsl:with-param name="tdSpan" select="$tdSpan" />
			<xsl:with-param name="href" select="$href" />
			<xsl:with-param name="value" select="$value" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="reportCommonInfoRow">
		<xsl:call-template name="reportIterationNumber">
			<xsl:with-param name="withTitle">
				false
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="reportStartTime">
			<xsl:with-param name="withTitle">
				false
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="reportEndTime">
			<xsl:with-param name="withTitle">
				false
			</xsl:with-param>
		</xsl:call-template>
		<xsl:call-template name="reportExecutionTime">
			<xsl:with-param name="withTitle">
				false
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="reportStartTime">
		<xsl:param name="withTitle" select="true()" />
		<xsl:param name="withDate" select="false()" />
		<xsl:param name="name" />
		<xsl:variable name="time"
			select="if( $withDate ) then @startTime else substring-after( @startTime,'T' )" />
		<xsl:call-template name="reportInfo">
			<xsl:with-param name="name">
				<xsl:if test="$withTitle">
					<xsl:choose>
						<xsl:when test="string-length(string($name))!=0">
							<xsl:value-of select="$name" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$START_TIME_OUTPUT" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
			</xsl:with-param>
			<xsl:with-param name="value">
				<xsl:choose>
					<xsl:when test="string-length(string($time))!=0">
						<xsl:value-of select="$time" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$NOT_APPLICABLE" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="reportEndTime">
		<xsl:param name="withTitle" select="true()" />
		<xsl:param name="withDate" select="false()" />
		<xsl:param name="name" />
		<xsl:variable name="time"
			select="if( $withDate ) then @endTime else substring-after( @endTime,'T' )" />
		<xsl:call-template name="reportInfo">
			<xsl:with-param name="name">
				<xsl:if test="$withTitle=true()">
					<xsl:choose>
						<xsl:when test="string-length(string($name))!=0">
							<xsl:value-of select="$name" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$END_TIME_OUTPUT" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
			</xsl:with-param>
			<xsl:with-param name="value">
				<xsl:choose>
					<xsl:when test="string-length($time)!=0">
						<xsl:value-of select="$time" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$NOT_APPLICABLE" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="reportExecutionTime">
		<xsl:param name="withTitle" select="true()" />
		<xsl:call-template name="copyAttribute">
			<xsl:with-param name="attribute" select="$EXECUTION_TIME" />
			<xsl:with-param name="outputAttribute" select="$EXECUTION_TIME_OUTPUT" />
			<xsl:with-param name="withTitle" select="$withTitle" />
			<xsl:with-param name="class" select="'figure'" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="addSectionHeader">
		<xsl:param name="header" />
		<h2>
			<a>
				<xsl:attribute name="name" select="generate-id()" />
				<xsl:value-of select="$header" />
			</a>
			<xsl:call-template name="createPrevLink" />
			<xsl:call-template name="createNextLink" />
		</h2>
	</xsl:template>

	<xsl:template name="createNextLink">
		<xsl:if test="following-sibling::*[name()=current()/name()][1]">
			<a>
				<xsl:attribute name="href"
					select="concat('#',generate-id( following-sibling::*[name()=current()/name()][1] ))" />
				<xsl:value-of select="$NEXT" />
			</a>
		</xsl:if>
	</xsl:template>

	<xsl:template name="createPrevLink">
		<xsl:if test="preceding-sibling::*[name()=current()/name()][1]">
			<a>
				<xsl:attribute name="href"
					select="concat('#',generate-id( preceding-sibling::*[name()=current()/name()][1] ))" />
				<xsl:value-of select="$PREV" />
			</a>
		</xsl:if>
	</xsl:template>

	<xsl:template name="addActionInfo">
		<xsl:param name="name" />
		<xsl:param name="class" />
		<xsl:variable name="type">
			<xsl:apply-templates select="." mode="label" />
		</xsl:variable>
		<xsl:variable name="title">
			<xsl:choose>
				<xsl:when test="string-length($name)!=0">
					<xsl:value-of select="$name" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@technical" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<h3>
			<xsl:if test="string-length($class)!=0">
				<xsl:attribute name="class" select="$class" />
			</xsl:if>
			<a>
				<xsl:attribute name="name" select="generate-id()" />
				<xsl:value-of select="concat($type,': ',$title)" />
			</a>
		</h3>
	</xsl:template>

	<xsl:template name="generateReportTitle">
			<h1>
				<xsl:value-of select="x:getTitle()" />
			</h1>
			<table class="faux">
				<xsl:if test="string-length(/Test/@version)!=0">
					<tr>
						<th>
							<xsl:value-of select="$detailedReport.version" />
						</th>
						<td>
							<xsl:call-template name="getVersion" />
						</td>
					</tr>
				</xsl:if>
				<xsl:if test="string-length(/Test/@testPath)!=0">
					<tr>
						<th>
							<xsl:value-of select="$detailedReport.test" />
						</th>
						<td>
							<xsl:call-template name="getTestID" />
						</td>
					</tr>
				</xsl:if>
				<xsl:if test="string-length(/Test/environment/@name)!=0">
					<tr>
						<th>
							<xsl:value-of select="$detailedReport.environment" />
						</th>
						<td>
							<xsl:call-template name="getEnvironmentName" />
						</td>
					</tr>
				</xsl:if>
				<xsl:variable name="statusKey">
					<xsl:call-template name="getOverallStatus" />
				</xsl:variable>
				<xsl:if test="string-length($statusKey)!=0">
					<tr>
						<th>
							<xsl:value-of select="$detailedReport.overallStatus" />
						</th>
						<td>
							<xsl:attribute name="class">
                                <xsl:value-of select="lower-case($statusKey)" />
                            </xsl:attribute>
							<xsl:call-template name="getOverallStatusString">
								<xsl:with-param name="statusKey" select="$statusKey" />
							</xsl:call-template>
						</td>
					</tr>
				</xsl:if>
				<tr>
					<xsl:call-template name="reportTestExecutionTime" />
				</tr>
			</table>
	</xsl:template>

	<xsl:template name="getVersion">
		<xsl:value-of select="/Test/@version" />
	</xsl:template>

	<xsl:template name="getEnvironmentName">
		<xsl:value-of select="/Test/environment/@name" />
	</xsl:template>

	<xsl:template name="getTestID">
		<xsl:value-of select="/Test/@testPath" />
	</xsl:template>

	<xsl:template name="getOverallStatus">
		<xsl:param name="id" />
		<xsl:value-of select="gfn:notBlank(/Test/summary/@overallStatus)" />
	</xsl:template>

	<xsl:template name="reportTestInfo">
		<xsl:for-each select="runProfileProperties">
			<xsl:call-template name="reportDescription">
				<xsl:with-param name="description" select="'runSettingsTestDescription'" />
			</xsl:call-template>
		</xsl:for-each>
		<tr>
			<xsl:call-template name="reportTestStartTime" />
			<xsl:call-template name="reportErrorMessages" />
		</tr>
		<tr>
			<xsl:call-template name="reportTestEndTime" />
			<xsl:call-template name="reportInfo">
				<xsl:with-param name="name" select="$detailedReport.warnings" />
				<xsl:with-param name="value">
					<xsl:call-template name="getWarningMessages" />
				</xsl:with-param>
			</xsl:call-template>
		</tr>
	</xsl:template>

	<xsl:template name="reportTestStartTime">
		<xsl:param name="useTitle" select="1" />
		<xsl:variable name="title">
			<xsl:if test="$useTitle=1">
				<xsl:value-of select="$START_TIME_OUTPUT" />
			</xsl:if>
		</xsl:variable>
		<xsl:call-template name="reportInfo">
			<xsl:with-param name="name" select="$title" />
			<xsl:with-param name="value"
				select="translate(/Test/summary/@startTime,'T',' ')" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="reportTestEndTime">
		<xsl:param name="useTitle" select="1" />
		<xsl:variable name="title">
			<xsl:if test="$useTitle=1">
				<xsl:value-of select="$END_TIME_OUTPUT" />
			</xsl:if>
		</xsl:variable>
		<xsl:call-template name="reportInfo">
			<xsl:with-param name="name" select="$title" />
			<xsl:with-param name="value"
				select="translate(/Test/summary/@endTime,'T',' ')" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="reportTestExecutionTime">
		<xsl:param name="useTitle" select="1" />
		<xsl:variable name="title">
			<xsl:if test="$useTitle=1">
				<xsl:value-of select="$EXECUTION_TIME_OUTPUT" />
			</xsl:if>
		</xsl:variable>
		<xsl:variable name="execTime">
			<xsl:call-template name="getTestExecutionTime" />
		</xsl:variable>
		<xsl:call-template name="reportInfo">
			<xsl:with-param name="name" select="$title" />
			<xsl:with-param name="value" select="$execTime" />
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="getTestExecutionTime">
		<xsl:variable name="id">
			<xsl:value-of select="gfn:notBlank(@testPath)" />
		</xsl:variable>
		<xsl:for-each select="/Test/summary">
			<xsl:value-of select="gfn:notBlank(@*[name()=$EXECUTION_TIME])" />
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="getErrorMessages">
		<xsl:param name="id" />
		<xsl:value-of select="gfn:notBlank(/Test/summary/@errorMessages)" />
	</xsl:template>

	<xsl:template name="reportErrorMessages">
		<xsl:param name="useTitle" select="true()" />
		<xsl:variable name="title">
			<xsl:if test="$useTitle">
				<xsl:value-of select="$detailedReport.errors" />
			</xsl:if>
		</xsl:variable>
		<xsl:call-template name="reportInfo">
			<xsl:with-param name="name" select="$title" />
			<xsl:with-param name="value">
				<xsl:call-template name="getErrorMessages" />
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>

	<xsl:template name="getWarningMessages">
		<xsl:param name="id" />
		<xsl:value-of select="gfn:notBlank(/Test/summary/@warningMessages)" />
	</xsl:template>

	<xsl:template name="getStatusCount">
		<xsl:param name="statusType" />
		<xsl:variable name="statusCount">
			<xsl:call-template name="getChildCountByAttribute">
				<xsl:with-param name="name" select="$STATUS" />
				<xsl:with-param name="value" select="$statusType" />
				<xsl:with-param name="incAttribute" select="id" />
				<xsl:with-param name="incValue">
					<xsl:call-template name="getTestID" />
				</xsl:with-param>
			</xsl:call-template>
		</xsl:variable>


		<xsl:variable name="iterationCount">
			<xsl:call-template name="getIterationCount" />
		</xsl:variable>

		<xsl:value-of
			select="gfn:formatString('{1} ({2})',$statusCount,gfn:getPercentage($statusCount,$iterationCount))" />
	</xsl:template>

	<xsl:template name="reportHostID">
		<xsl:param name="withTitle" select="true()" />
		<xsl:call-template name="copyAttribute">
			<xsl:with-param name="attribute" select="$HOST_ID" />
			<xsl:with-param name="outputAttribute" select="$HOST_ID_OUTPUT" />
			<xsl:with-param name="withTitle" select="$withTitle" />
		</xsl:call-template>
	</xsl:template>

	<!-- Other xslts can override this behaviour by specifying a template matching
		their desired action, using mode="reportStatus" and providing a higher priority.
		See assertTrue.xsl for an example. -->
	<xsl:template match="*" mode="reportStatus" priority="1">
		<td>
			<xsl:choose>
				<xsl:when test="count(@*[name()=$EXECUTION_CANCELLED])">
					<xsl:value-of select="$CANCEL_STATUS" />
				</xsl:when>
				<xsl:otherwise>

					<xsl:variable name="statusNode"
						select="
                if  (: Check if the next element is a Validate element :)
                    (following-sibling::*[1][name()=$VALIDATE])
                then
                	(: if so get it :)
                    following-sibling::*[1][name()=$VALIDATE]
                else
                	(: else use the current element:)
                    current()" />

					<xsl:choose>
						<xsl:when test="$statusNode/@iterationStatus">
							<xsl:attribute name="class"
								select="lower-case($statusNode/@iterationStatus)" />
							<xsl:call-template name="getOverallStatusString">
								<xsl:with-param name="statusKey"
									select="$statusNode/@iterationStatus" />
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$NOT_APPLICABLE" />
						</xsl:otherwise>
					</xsl:choose>

				</xsl:otherwise>
			</xsl:choose>
		</td>
	</xsl:template>

	<xsl:template match="*[@iterationStatus='Skipped']"
		priority="2" />

	<xsl:template match="prepare|execute|cleanup">
		<xsl:param name="iterOnly" />
		<xsl:call-template name="addSectionHeader">
			<xsl:with-param name="header">
				<xsl:choose>
					<xsl:when test="name() = 'prepare'">
						<xsl:value-of select="$detailedReport.initialise" />
					</xsl:when>
					<xsl:when test="name() = 'execute'">
						<xsl:value-of select="$detailedReport.testSteps" />
					</xsl:when>
					<xsl:when test="name() = 'cleanup'">
						<xsl:value-of select="'detailedReport.tearDown'" />
					</xsl:when>
				</xsl:choose>
			</xsl:with-param>
		</xsl:call-template>
		<table class="faux">
			<xsl:call-template name="reportCommonInfo">
				<xsl:with-param name="host" select="true()" />
			</xsl:call-template>
		</table>

		<xsl:apply-templates select="*" />
	</xsl:template>

	<xsl:template name="reportTask">
		<tr>
			<xsl:call-template name="reportInfo">
				<xsl:with-param name="value" select="@*[name()=$TASK]" />
				<xsl:with-param name="name" select="$TASK_OUTPUT" />
				<xsl:with-param name="tdSpan" select="4" />
			</xsl:call-template>
		</tr>
	</xsl:template>

	<xsl:template match="validate" mode="label">
		<xsl:value-of select="$VALIDATE_OUTPUT" />
	</xsl:template>

	<xsl:template match="validate">
		<xsl:call-template name="addActionInfo" />
		<table class="faux">
			<xsl:call-template name="reportCommonInfo">
				<xsl:with-param name="validation" select="true()" />
			</xsl:call-template>
		</table>
	</xsl:template>

	<xsl:template name="reportValidationError">
		<xsl:if test="count(@*[name()=$ERROR])!=0">
			<tr>
				<xsl:call-template name="reportInfo">
					<xsl:with-param name="name" select="$ERROR_OUTPUT" />
					<xsl:with-param name="value" select="@*[name()=$ERROR]" />
					<xsl:with-param name="tdSpan" select="4" />
				</xsl:call-template>
			</tr>
			<xsl:if test="count(*[name()=$VALIDATION_MESSAGE])!=0">
				<xsl:for-each select="*[name()=$VALIDATION_MESSAGE]">
					<tr>
						<xsl:call-template name="reportInfo">
							<xsl:with-param name="name" select="$VALIDATION_MESSAGE_OUTPUT" />
							<xsl:with-param name="value" select="text()" />
							<xsl:with-param name="tdSpan" select="4" />
						</xsl:call-template>
					</tr>
				</xsl:for-each>
			</xsl:if>
		</xsl:if>
	</xsl:template>

	<xsl:template name="createSummaryTable">
		<table>
			<tr>
				<th>
					<xsl:value-of select="$ACTIONS_OUTPUT" />
				</th>
				<th>
					<xsl:value-of select="$START_TIME_OUTPUT" />
				</th>
				<th>
					<xsl:value-of select="$END_TIME_OUTPUT" />
				</th>
				<th>
					<xsl:value-of select="$EXECUTION_TIME_OUTPUT" />
				</th>
				<th>
					<xsl:value-of select="$STATUS_OUTPUT" />
				</th>
			</tr>
			<xsl:apply-templates select="*" mode="summary" />
		</table>
	</xsl:template>

	<xsl:template match="error">
		<tr>
			<td>
				<xsl:call-template name="createSummaryRowA">
					<xsl:with-param name="name">
						<xsl:value-of select="@actionId" />
					</xsl:with-param>
				</xsl:call-template>
			</td>
			<td>
				<xsl:for-each select="msgLine">
					<xsl:value-of select="text()" />
					<br />
				</xsl:for-each>
			</td>
		</tr>
	</xsl:template>

	<xsl:template name="processSummaryRow">
		<xsl:param name="name" />
		<xsl:param name="label" select="true()" />

		<xsl:choose>
			<xsl:when test="parent::*/parent::Filter">
				<!-- this is a bit of a hack. For each is the only way to change the
					context node. When dealing with a filtered action the parent filter element
					has the information eg iterationStatus. -->
				<xsl:for-each select="parent::*/parent::Filter">
					<xsl:call-template name="doSummaryRow">
						<xsl:with-param name="name" select="$name" />
						<xsl:with-param name="label" select="$label" />
					</xsl:call-template>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="doSummaryRow">
					<xsl:with-param name="name" select="$name" />
					<xsl:with-param name="label" select="$label" />
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="doSummaryRow">
		<xsl:param name="name" />
		<xsl:param name="label" select="true()" />
		<tr>
			<td>
				<xsl:attribute name="style"
					select="concat('padding-left: ', x:indentation(current()) * 10, 'px; text-align: left')" />
				<xsl:call-template name="createSummaryRowA">
					<xsl:with-param name="name" select="$name" />
					<xsl:with-param name="label" select="$label" />
				</xsl:call-template>
			</td>
			<xsl:call-template name="reportStartTime">
				<xsl:with-param name="withTitle" select="false()" />
			</xsl:call-template>
			<xsl:call-template name="reportEndTime">
				<xsl:with-param name="withTitle" select="false()" />
			</xsl:call-template>
			<xsl:call-template name="reportExecutionTime">
				<xsl:with-param name="withTitle" select="false()" />
			</xsl:call-template>
			<xsl:if test="string-length(@*[name()=$STATUS])!=0">
				<!-- reportStatus is overridable so invoke templates using apply-templates -->
				<xsl:apply-templates select="." mode="reportStatus" />
			</xsl:if>
		</tr>
	</xsl:template>

	<xsl:template name="createSummaryRowA">
		<xsl:param name="name" />
		<xsl:param name="label" select="false()" />
		<xsl:variable name="title">
			<xsl:choose>
				<xsl:when test="string-length($name)!=0">
					<xsl:choose>
						<xsl:when test="$name=string('Iteration#Multiple Instance')">
							<xsl:variable name="instCount"
								select="/*/runProfileProperties/@performanceInstances" />
							<xsl:value-of select="$instCount" />
							<xsl:choose>
								<xsl:when test="number($instCount)=1">
									<xsl:value-of select="string(' instance of ')" />
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="string(' instances of ')" />
								</xsl:otherwise>
							</xsl:choose>
							<xsl:call-template name="getTestID" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$name" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="@technical" />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<a>
			<xsl:attribute name="href" select="concat('#',generate-id())" />
			<xsl:if test="$label">
				<xsl:apply-templates select="." mode="label" />
				<xsl:value-of select="': '" />
			</xsl:if>
			<xsl:value-of select="$title" />
		</a>
	</xsl:template>

	<xsl:template name="processEnvironment">
		<xsl:for-each select="child::environment[1]">
			<xsl:call-template name="addSectionHeader">
				<xsl:with-param name="header"
					select="$detailedReport.environmentHeading" />
			</xsl:call-template>
			<table class="faux">
				<tr>
					<xsl:call-template name="copyAttribute">
						<xsl:with-param name="attribute" select="$NAME" />
						<xsl:with-param name="outputAttribute" select="$NAME_OUTPUT" />
					</xsl:call-template>
				</tr>
			</table>
			<xsl:call-template name="processTagDataStore">
				<xsl:with-param name="elementName" select="'environment'" />
				<xsl:with-param name="tableName" select="$ENVIRONMENT_TAGS_OUTPUT" />
			</xsl:call-template>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="processTagDataStore">
		<xsl:param name="elementName" />
		<xsl:param name="tableName" />
		<xsl:for-each select="descendant-or-self::*[name()=$elementName][1]">
			<xsl:if test="*[name()=$TAG]">
				<xsl:call-template name="addSectionHeader">
					<xsl:with-param name="header" select="$tableName" />
				</xsl:call-template>
				<table>
					<tr>
						<th>
							<xsl:value-of select="$NAME_OUTPUT" />
						</th>
						<th>
							<xsl:value-of select="$VALUE_OUTPUT" />
						</th>
					</tr>
					<xsl:for-each select="*[name()=$TAG]">
						<tr>
							<td>
								<xsl:value-of select="@name" />
							</td>
							<td>
								<xsl:choose>
									<xsl:when test="@isMessage = true()">
										<xsl:value-of select="$MESSAGE_TAG" />
									</xsl:when>
									<xsl:when test="@value">
										<xsl:value-of select="@value" />
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="value/text()" />
									</xsl:otherwise>
								</xsl:choose>
							</td>
						</tr>
					</xsl:for-each>
				</table>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="formatNewLines">
		<xsl:param name="text" select="." />
		<xsl:choose>
			<xsl:when test="contains($text,'&#10;')">
				<xsl:value-of select="substring-before($text,'&#10;')" />
				<br />
				<xsl:call-template name="formatNewLines">
					<xsl:with-param name="text"
						select="substring-after($text,'&#10;')" />
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$text" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Selects the output string for known statuses. If the status is not
		known then it is selected as a fallback -->
	<xsl:template name="getOverallStatusString">
		<xsl:param name="statusKey" />
		<xsl:if test="$statusKey = $PASSED">
			<xsl:value-of select="$PASSED_OUTPUT" />
		</xsl:if>
		<xsl:if test="$statusKey = $FAILED">
			<xsl:value-of select="$FAILED_OUTPUT" />
		</xsl:if>
		<xsl:if test="$statusKey = $CANCELLED">
			<xsl:value-of select="$CANCELLED_OUTPUT" />
		</xsl:if>

		<!-- As a fallback just print out the statusKey. -->
		<xsl:if
			test="$statusKey != $PASSED and $statusKey != $FAILED and $statusKey != $CANCELLED">
			<xsl:value-of select="$statusKey" />
		</xsl:if>
	</xsl:template>

	<xsl:template match="*">
		<div>
			<xsl:value-of select="name()" />
		</div>
	</xsl:template>

	<xsl:template match="*" mode="summary">
		<xsl:apply-templates select="*" mode="summary" />
	</xsl:template>

	<xsl:template match="environment" />
	<xsl:template match="summary" />
	<xsl:template match="tagValues" />
</xsl:stylesheet>